# coding: utf-8

"""
    Universal DDI Anycast API

    Anycast capability enables HA (High Availability) configuration of Universal DDI applications that run on equipment located on customer's premises (on-prem hosts). Anycast supports DNS, as well as DNS-forwarding services.  Anycast-enabled application setups use multiple on-premises installations for one particular application type. Multiple application instances are configured to use the same endpoint address. Anycast capability is collocated with such application instance, monitoring the local application instance and advertising to the upstream router (a customer equipment) a per-instance, local route to the common application endpoint address, as long as the local application instance is available. Depending on the type of the upstream router, the customer may configure local route advertisement via either BGP (Boarder Gateway Protocol) or OSPF (Open Shortest Path First) routing protocols. Both protocols may be enabled as well. Multiple routes to the common application service address provide redundancy without the need to reconfigure application clients.  Should an application instance become unavailable, the local route advertisements stop, resulting in withdrawal of the route (in the upstream router) to the application instance that has gone out of service and ensuring that subsequent application requests thus get routed to the remaining available application instances.  

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self


class OspfConfig(BaseModel):
    """
    OspfConfig
    """

  # noqa: E501
    area: Optional[StrictStr] = Field(
        default=None,
        description=
        "OSPF area identifier; usually in the format of an IPv4 address (although not an address itself)"
    )
    area_type: Optional[StrictStr] = None
    authentication_key: Optional[StrictStr] = None
    authentication_key_id: Optional[StrictInt] = None
    authentication_type: Optional[StrictStr] = None
    cost: Optional[StrictInt] = None
    dead_interval: Optional[StrictInt] = None
    hello_interval: Optional[StrictInt] = None
    interface: Optional[StrictStr] = Field(
        default=None,
        description=
        "Name of the interface that is configured with external IP address of the host"
    )
    preamble: Optional[StrictStr] = Field(
        default=None,
        description=
        "Any predefined OSPF configuration, with embedded new lines; the preamble will be prepended to the generated BGP configuration."
    )
    retransmit_interval: Optional[StrictInt] = None
    transmit_delay: Optional[StrictInt] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "area", "area_type", "authentication_key", "authentication_key_id",
        "authentication_type", "cost", "dead_interval", "hello_interval",
        "interface", "preamble", "retransmit_interval", "transmit_delay"
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of OspfConfig from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of OspfConfig from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "area":
            obj.get("area"),
            "area_type":
            obj.get("area_type"),
            "authentication_key":
            obj.get("authentication_key"),
            "authentication_key_id":
            obj.get("authentication_key_id"),
            "authentication_type":
            obj.get("authentication_type"),
            "cost":
            obj.get("cost"),
            "dead_interval":
            obj.get("dead_interval"),
            "hello_interval":
            obj.get("hello_interval"),
            "interface":
            obj.get("interface"),
            "preamble":
            obj.get("preamble"),
            "retransmit_interval":
            obj.get("retransmit_interval"),
            "transmit_delay":
            obj.get("transmit_delay")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj
