# coding: utf-8

"""
    IP Address Management API

    The IPAM/DHCP Application is a Universal DDI service providing IP address management and DHCP protocol features. The IPAM component provides visibility into and provisioning tools to manage networking spaces, monitoring and reporting of entire IP address infrastructures, and integration with DNS and DHCP protocols. The DHCP component provides DHCP protocol configuration service with on-prem host serving DHCP protocol. It is part of the full-featured, DDI cloud solution that enables customers to deploy large numbers of protocol servers to deliver DNS and DHCP throughout their enterprise network.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from ipam.models.ha_group_heartbeats import HAGroupHeartbeats
from typing import Optional, Set
from typing_extensions import Self


class HAGroupHost(BaseModel):
    """
    An __HAGroupHost__ object (_dhcp/ha_group_host_) represents an on-prem host belonging to an HA Group.
    """ # noqa: E501
    address: Optional[StrictStr] = Field(
        default=None, description="The address on which this host listens.")
    heartbeats: Optional[List[HAGroupHeartbeats]] = Field(
        default=None,
        description=
        "Last successful heartbeat received from its peer/s. This field is set when the _collect_stats_ is set to _true_ in the _GET_ _/dhcp/ha_group_ request."
    )
    host: StrictStr = Field(description="The resource identifier.")
    port: Optional[StrictInt] = Field(default=None, description="The HA port.")
    port_v6: Optional[StrictInt] = Field(
        default=None, description="The HA port used for IPv6 communication.")
    role: Optional[StrictStr] = Field(
        default=None,
        description=
        "The role of this host in the HA relationship: _active_ or _passive_.")
    state: Optional[StrictStr] = Field(
        default=None,
        description=
        "The state of DHCP on the host. This field is set when the _collect_stats_ is set to _true_ in the _GET_ _/dhcp/ha_group_ request."
    )
    state_v6: Optional[StrictStr] = Field(
        default=None,
        description=
        "The state of DHCPv6 on the host. This field is set when the _collect_stats_ is set to _true_ in the _GET_ _/dhcp/ha_group_ request."
    )
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "address", "heartbeats", "host", "port", "port_v6", "role", "state",
        "state_v6"
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of HAGroupHost from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "port",
            "port_v6",
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in heartbeats (list)
        _items = []
        if self.heartbeats:
            for _item in self.heartbeats:
                if _item:
                    _items.append(_item.to_dict())
            _dict['heartbeats'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of HAGroupHost from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "address":
            obj.get("address"),
            "heartbeats": [
                HAGroupHeartbeats.from_dict(_item)
                for _item in obj["heartbeats"]
            ] if obj.get("heartbeats") is not None else None,
            "host":
            obj.get("host"),
            "port":
            obj.get("port"),
            "port_v6":
            obj.get("port_v6"),
            "role":
            obj.get("role"),
            "state":
            obj.get("state"),
            "state_v6":
            obj.get("state_v6")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj
